@@ -5,7 +5,9 @@ use strict;
 use Carp;
 use Exporter 'import';  # just use the import() function, without the rest of the overhead of ISA
 
-use version 0.77; our $VERSION = version->declare('0.013_006');
+use version 0.77; our $VERSION = version->declare('v0.14003.0');    # TODO: when moving to 1.0, use '1.000' again
+    # only using v-notation because of a typo in Makefile.PL which bumped up from 0.014 to v0.14001.0, so now I have to
+    # do sub-versions above 14k, which can only be done in v-notation.
 
 =pod
 
@@ -117,6 +119,10 @@ a transition to v2.
 
 =item v0.013_003: C<nextafter()> renamed to C<nextAfter()>
 
+=item v0.013_008: C<absolute()> renamed to C<abs()>, and noted that perl's builtin can be accessed via C<CORE::abs()>
+
+=item v0.14001: messed up version numbering convention when I get to 1.000, it will be reset to decimal-based.
+
 =back
 
 =head1 EXPORTABLE FUNCTIONS AND VARIABLES
@@ -145,8 +151,8 @@ my  @EXPORT_CONST = qw(
 );
 my @EXPORT_INFO = qw(isSignMinus isNormal isFinite isZero isSubnormal
     isInfinite isNaN isSignaling isSignalingConvertedToQuiet isCanonical
-    class radix totalOrder totalOrderMag);
-my @EXPORT_SIGNBIT = qw(negate absolute copySign isSignMinus);
+    class radix totalOrder totalOrderMag compareFloatingValue compareFloatingMag);
+my @EXPORT_SIGNBIT = qw(copy negate abs copySign isSignMinus);
 
 our @EXPORT_OK = (@EXPORT_FLOATING, @EXPORT_RAW754, @EXPORT_ULP, @EXPORT_CONST, @EXPORT_INFO, @EXPORT_SIGNBIT);
 our %EXPORT_TAGS = (
@@ -389,9 +395,9 @@ zeroes, infinities, a variety of signaling and quiet NAN values.
     POS_NORM_SMALLEST    # +0x1.0000000000000p-1022  # smallest positive value that allows for normal representation in 64bit floating-point
     POS_NORM_BIGGEST     # +0x1.fffffffffffffp+1023  # largest positive value that allows for normal representation in 64bit floating-point
     POS_INF              # +0x1.#INF000000000p+0000  # positive infinity: indicates that the answer is out of the range of a 64bit floating-point
-    POS_SNAN_FIRST       # +0x1.#SNAN00000000p+0000  # positive signaling NAN with "0x0000000000001" as the system-dependent information; note that many perl interpreters will internally convert this to a Quiet NaN (QNAN)
-    POS_SNAN_LAST        # +0x1.#SNAN00000000p+0000  # positive signaling NAN with "0x7FFFFFFFFFFFF" as the system-dependent information; note that many perl interpreters will internally convert this to a Quiet NaN (QNAN)
-    POS_IND              # +0x1.#QNAN00000000p+0000  # positive quiet NAN with "0x8000000000000" as the system-dependent information; some perl interpreters define the NEG_IND as an "indeterminate" value (IND), and it wouldn't surprise me if some also defined this positive version as "indeterminate" as well
+    POS_SNAN_FIRST       # +0x1.#SNAN00000000p+0000  # positive signaling NAN with "0x0000000000001" as the system-dependent information [*]
+    POS_SNAN_LAST        # +0x1.#SNAN00000000p+0000  # positive signaling NAN with "0x7FFFFFFFFFFFF" as the system-dependent information [*]
+    POS_IND              # +0x1.#QNAN00000000p+0000  # positive quiet NAN with "0x8000000000000" as the system-dependent information [%]
     POS_QNAN_FIRST       # +0x1.#QNAN00000000p+0000  # positive quiet NAN with "0x8000000000001" as the system-dependent information
     POS_QNAN_LAST        # +0x1.#QNAN00000000p+0000  # positive quiet NAN with "0xFFFFFFFFFFFFF" as the system-dependent information
 
@@ -401,12 +407,16 @@ zeroes, infinities, a variety of signaling and quiet NAN values.
     NEG_NORM_SMALLEST    # -0x1.0000000000000p-1022  # smallest negative value that allows for normal representation in 64bit floating-point
     NEG_NORM_BIGGEST     # -0x1.fffffffffffffp+1023  # largest negative value that allows for normal representation in 64bit floating-point
     NEG_INF              # -0x1.#INF000000000p+0000  # negative infinity: indicates that the answer is out of the range of a 64bit floating-point
-    NEG_SNAN_FIRST       # -0x1.#SNAN00000000p+0000  # negative signaling NAN with "0x0000000000001" as the system-dependent information; note that many perl interpreters will internally convert this to a Quiet NaN (QNAN)
-    NEG_SNAN_LAST        # -0x1.#SNAN00000000p+0000  # negative signaling NAN with "0x7FFFFFFFFFFFF" as the system-dependent information; note that many perl interpreters will internally convert this to a Quiet NaN (QNAN)
-    NEG_IND              # -0x1.#IND000000000p+0000  # negative quiet NAN with "0x8000000000000" as the system-dependent information; some perl interpreters define the NEG_IND as an "indeterminate" value (IND)
+    NEG_SNAN_FIRST       # -0x1.#SNAN00000000p+0000  # negative signaling NAN with "0x0000000000001" as the system-dependent information [*]
+    NEG_SNAN_LAST        # -0x1.#SNAN00000000p+0000  # negative signaling NAN with "0x7FFFFFFFFFFFF" as the system-dependent information [*]
+    NEG_IND              # -0x1.#IND000000000p+0000  # negative quiet NAN with "0x8000000000000" as the system-dependent information [%]
     NEG_QNAN_FIRST       # -0x1.#QNAN00000000p+0000  # negative quiet NAN with "0x8000000000001" as the system-dependent information
     NEG_QNAN_LAST        # -0x1.#QNAN00000000p+0000  # negative quiet NAN with "0xFFFFFFFFFFFFF" as the system-dependent information
 
+	[*] note that many perl interpreters will internally convert Signalling NaN (SNAN) to Quiet NaN (QNAN)
+	[%] some perl interpreters define the zeroeth negative Quiet NaN, NEG_IND, as an "indeterminate" value (IND);
+	    in a symmetrical world, they would also define the zeroeth positive Quiet NaN, POS_IND, as an "indeterminate" value (IND)
+
 =cut
 
 { my $local; sub POS_ZERO           () { $local = hexstr754_to_double('000'.'0000000000000') unless defined $local; return $local; } }
@@ -672,7 +682,7 @@ sub isSignaling {
     my $exp = substr($h,0,3);
     my $frc = substr($h,3,13);
     my $qbit = (0x8 && hex(substr($h,3,1))) >> 3;   # 1: quiet, 0: signaling
-    return ($exp eq '7FF' || $exp eq 'FFF') && ($frc ne '0'x13)  && (!$qbit) || 0;
+    return ($exp eq '7FF' || $exp eq 'FFF') && ($frc ne '0'x13)  && (!$qbit) || 0;  # v0.013_007 = possible coverage bug: don't know whether it's the paren or non-paren, but the "LEFT=TRUE" condition of "OR 2 CONDITIONS" is never covered
 }
 
 =head4 isSignalingConvertedToQuiet()
@@ -685,7 +695,7 @@ function is meaningful in your implementation of perl.
 =cut
 
 sub isSignalingConvertedToQuiet {
-    !isSignaling( POS_SNAN_FIRST ) || 0
+    !isSignaling( POS_SNAN_FIRST ) || 0     # v0.013 coverage note: ignore Devel::Cover failures on this line
 }
 
 =head3 isCanonical( I<value> )
@@ -721,16 +731,16 @@ Returns the "class" of the I<value>:
 =cut
 
 sub class {
-    return 'signalingNaN'       if isSignaling($_[0]);
+    return 'signalingNaN'       if isSignaling($_[0]);      # v0.013 coverage note: ignore Devel::Cover failures on this line (won't return on systems that quiet SNaNs
     return 'quietNaN'           if isNaN($_[0]);
     return 'negativeInfinity'   if isInfinite($_[0])    && isSignMinus($_[0]);
     return 'negativeNormal'     if isNormal($_[0])      && isSignMinus($_[0]);
     return 'negativeSubnormal'  if isSubnormal($_[0])   && isSignMinus($_[0]);
     return 'negativeZero'       if isZero($_[0])        && isSignMinus($_[0]);
-    return 'positiveZero'       if isZero($_[0])        && !isSignMinus($_[0]);
-    return 'positiveSubnormal'  if isSubnormal($_[0])   && !isSignMinus($_[0]);
-    return 'positiveNormal'     if isNormal($_[0])      && !isSignMinus($_[0]);
-    return 'positiveInfinity'   if isInfinite($_[0])    && !isSignMinus($_[0]);
+    return 'positiveZero'       if isZero($_[0])        && !isSignMinus($_[0]);     # v0.013 coverage note: ignore Devel::Cover->CONDITION failure; alternate condition already returned above
+    return 'positiveSubnormal'  if isSubnormal($_[0])   && !isSignMinus($_[0]);     # v0.013 coverage note: ignore Devel::Cover->CONDITION failure; alternate condition already returned above
+    return 'positiveNormal'     if isNormal($_[0])      && !isSignMinus($_[0]);     # v0.013 coverage note: ignore Devel::Cover->CONDITION failure; alternate condition already returned above
+    return 'positiveInfinity'   if isInfinite($_[0])    && !isSignMinus($_[0]);     # v0.013 coverage note: no tests for FALSE because all conditions covered above
 }
 
 =head3 radix( I<value> )
@@ -800,13 +810,70 @@ sub totalOrderMag {
     return totalOrder( $x, $y );                                                        # compare normally
 }
 
-# TODO = spaceship() and spaceshipMag() similar to totalOrder and totalOrderMag, but with '<' => -1, '==' => 0, '>' => +1
+=head3 compareFloatingValue( I<x>, I<y> )
+
+=head3 compareFloatingMag( I<x>, I<y> )
+
+These are similar to C<totalOrder()> and C<totalOrderMag()>, except they return
+-1 for C<x E<lt> y>, 0 for C<x == y>, and +1 for C<x E<gt> y>.
+
+These are not in IEEE 754-2008, but are included as functions to replace the perl spaceship
+(C<E<lt>=E<gt>>) when comparing floating-point values that might be NaN.
+
+=cut
+
+sub compareFloatingValue {
+    my ($x, $y) = @_[0,1];
+    my ($bx,$by) = map { binstr754_from_double($_) } $x, $y;        # convert to binary strings
+    my @xsegs = ($bx =~ /(.)(.{11})(.{20})(.{32})/);                # split into sign, exponent, MSB, LSB
+    my @ysegs = ($by =~ /(.)(.{11})(.{20})(.{32})/);                # split into sign, exponent, MSB, LSB
+    my ($xin, $yin) = map { isNaN($_) } $x, $y;                     # determine if NaN: used twice each, so save the values rather than running twice each during if-switch
+
+    if( $xin && $yin ) {                                            # BOTH NaN
+        # use a trick: the rules for both-NaN treat it as if it's just another floating point,
+        #  so lie about the exponent and do a normal comparison
+        ($bx, $by) = map { $_->[1] = '1' . '0'x10; join '', @$_ } \@xsegs, \@ysegs;
+        ($x, $y) = map { binstr754_to_double($_) } $bx, $by;
+        return ($x <=> $y);
+    } elsif ( $xin ) {                                              # just x NaN: if isNaN(x) && isNegative(x) THEN -1 (x<y) ELSE (x>y)
+        return ( ($xsegs[0])*-1 || +1 );
+    } elsif ( $yin ) {                                              # just y NaN: if isNaN(y) && !isNegative(y) THEN -1 (x<y) ELSE (x>y)
+        return ( (!$ysegs[0])*-1 || +1 );
+    } elsif ( isZero($x) && isZero($y) ) {                          # both zero: TRUE if x NEG, or if x==y
+        # trick = -signbit(x) <=> -signbit(y), since signbit is 1 for negative, -signbit = -1 for negative
+        return (-$xsegs[0] <=> -$ysegs[0]);
+    } else {                                                        # numeric comparison (works for inf, normal, subnormal, or only one +/-zero)
+        return ($x <=> $y);
+    }
+}
+
+sub compareFloatingMag {
+    my ($x, $y)     = @_[0,1];
+    my ($bx,$by)    = map { binstr754_from_double($_) } $x, $y;                         # convert to binary strings
+    ($x,  $y)       = map { substr $_, 0, 1, '0'; binstr754_to_double($_) } $bx, $by;   # set sign bit to 0, and convert back to number
+    return compareFloatingValue( $x, $y );                                              # compare normally
+}
 
 =head2 :signbit
 
 These functions, from IEEE Std 754-2008, manipulate the sign bits
 of the argument(s)set P.
 
+See IEEE Std 754-2008 #5.5.1 "Sign bit operations": This section asserts
+that the sign bit operations (including C<negate>, C<abs>, and C<copySign>)
+should only affect the sign bit, and should treat numbers and NaNs alike.
+
+=head3 copy( I<value> )
+
+Copies the I<value> to the output, leaving the sign bit unchanged, for all
+numbers and NaNs.
+
+=cut
+
+sub copy {
+	return shift;
+}
+
 =head3 negate( I<value> )
 
 Reverses the sign bit of I<value>.  (If the sign bit is set on I<value>,
@@ -822,23 +889,26 @@ sub negate {
     return binstr754_to_double($b);                                                     # convert to floating-point
 }
 
-=head3 CORE::abs( I<value> )
-=head3 absolute( I<value> )
+=head3 abs( I<value> )
+
+Similar to the C<CORE::abs()> builtin function, C<abs()> is provided as a
+module-based function to get the absolute value (magnitude) of a 64bit
+floating-point number.
 
 The C<CORE::abs()> function behaves properly (per the IEEE 754 description)
-for all classes of I<value> (including signed zeroes, infinities, and NaNs),
-per the installation test suite.
+for all classes of I<value>, except that many implementations do not correctly
+handle -NaN properly, outputting -NaN, which is in violation of the standard.
+The C<Data::IEEE754::Tools::abs()> function correctly treats NaNs in the same
+way it treats numerical values, and clears the sign bit on the output.
 
-However, C<absolute()> is provided as a module-based alternative.
-
-The test suite runs the expected input/output pairs thru both C<CORE::abs()> and
-C<Data::IEEE754::Tools::absolute()>, so if any of those fail on your system, or
-if you find an untested edge case where C<CORE::abs()> (or C<Data::IEEE754::Tools::absolute()>)
-do not behave as expected, please file a bug report.
+Please note that exporting C<abs()> or C<:signbit> from this module will
+"hide" the builtin C<abs()> function.  If you really need to use the builtin
+version (for example, you care more about execution speed than its ability to find
+the absolute value of a signed NaN), then you may call it as C<CORE::abs>.
 
 =cut
 
-sub absolute {
+sub abs {
     my $b = binstr754_from_double(shift);                                               # convert to binary string
     substr $b, 0, 1, '0';                                                               # replace sign
     return binstr754_to_double($b);                                                     # convert to floating-point
@@ -908,7 +978,8 @@ the sub.
 Peter C. Jones C<E<lt>petercj AT cpan DOT orgE<gt>>
 
 Please report any bugs or feature requests emailing C<E<lt>bug-Data-IEEE754-Tools AT rt.cpan.orgE<gt>>
-or thru the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Data-IEEE754-Tools>.
+or thru the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Data-IEEE754-Tools>,
+or thru the repository's interface at L<https://github.com/pryrt/Data-IEEE754-Tools/issues>.
 
 =head1 COPYRIGHT
 
